@@ -9,7 +9,6 @@ from typing import (
     ClassVar,
     Generic,
     Literal,
-    MutableMapping,
     NamedTuple,
     TypeVar,
     overload,
@@ -19,6 +18,7 @@ from ._expressions import Name
 
 if TYPE_CHECKING:
     import builtins
+    from collections.abc import MutableMapping
 
 T = TypeVar("T")
 A = TypeVar("A")
@@ -97,7 +97,8 @@ class ContextKey(Name, Generic[A, T]):
         *,
         id: str = "",  # optional because of __set_name__
     ) -> None:
-        super().__init__(id or "")
+        bound = type(default_value) if default_value is not MISSING else None
+        super().__init__(id or "", bound=bound)
         self._default_value = default_value
         self._getter = getter
         self._description = description
