@@ -1,13 +1,15 @@
 from __future__ import annotations
 
 from concurrent.futures import Future, ThreadPoolExecutor
-from typing import TYPE_CHECKING, Any, Callable, Generic, Iterator, TypeVar, cast
+from typing import TYPE_CHECKING, Any, Callable, Generic, TypeVar, cast
 
 from in_n_out import Store
 from psygnal import Signal
 
 # maintain runtime compatibility with older typing_extensions
 if TYPE_CHECKING:
+    from collections.abc import Iterator
+
     from typing_extensions import ParamSpec
 
     from app_model.types import Action, DisposeCallable
@@ -36,13 +38,13 @@ class RegisteredCommand(Generic[P, R]):
     """
 
     __slots__ = (
-        "id",
-        "callback",
-        "title",
-        "_resolved_callback",
-        "_injection_store",
-        "_injected_callback",
         "_initialized",
+        "_injected_callback",
+        "_injection_store",
+        "_resolved_callback",
+        "callback",
+        "id",
+        "title",
     )
 
     def __init__(
