@@ -1,5 +1,7 @@
 from __future__ import annotations
 
+from bisect import insort_left
+from collections import defaultdict
 from typing import TYPE_CHECKING, Callable, NamedTuple
 
 from psygnal import Signal
@@ -7,10 +9,16 @@ from psygnal import Signal
 from app_model.types import KeyBinding
 
 if TYPE_CHECKING:
-    from typing import Iterator, TypeVar
+    from collections.abc import Iterable, Iterator, Mapping
+    from typing import TypeVar
 
     from app_model import expressions
-    from app_model.types import Action, DisposeCallable, KeyBindingRule
+    from app_model.types import (
+        Action,
+        DisposeCallable,
+        KeyBindingRule,
+        KeyBindingSource,
+    )
 
     CommandDecorator = Callable[[Callable], Callable]
     CommandCallable = TypeVar("CommandCallable", bound=Callable)
@@ -22,8 +30,24 @@ class _RegisteredKeyBinding(NamedTuple):
     keybinding: KeyBinding  # the keycode to bind to
     command_id: str  # the command to run
     weight: int  # the weight of the binding, for prioritization
+    source: KeyBindingSource  # who defined the binding, for prioritization
     when: expressions.Expr | None = None  # condition to enable keybinding
 
+    def __gt__(self, other: object) -> bool:
+        if not isinstance(other, _RegisteredKeyBinding):
+            return NotImplemented
+        return (self.source, self.weight) > (other.source, other.weight)
+
+    def __lt__(self, other: object) -> bool:
+        if not isinstance(other, _RegisteredKeyBinding):
+            return NotImplemented
+        return (self.source, self.weight) < (other.source, other.weight)
+
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, _RegisteredKeyBinding):
+            return NotImplemented
+        return (self.source, self.weight) == (other.source, other.weight)
+
 
 class KeyBindingsRegistry:
     """Registry for keybindings.
@@ -37,11 +61,16 @@ class KeyBindingsRegistry:
     """
 
     registered = Signal()
+    unregistered = Signal()
 
     def __init__(self) -> None:
-        self._keybindings: list[_RegisteredKeyBinding] = []
+        self._keymap = defaultdict[int, list[_RegisteredKeyBinding]](list)
         self._filter_keybinding: Callable[[KeyBinding], str] | None = None
 
+    @property
+    def _keybindings(self) -> Iterable[_RegisteredKeyBinding]:
+        return (entry for entries in self._keymap.values() for entry in entries)
+
     @property
     def filter_keybinding(self) -> Callable[[KeyBinding], str] | None:
         """Return the `filter_keybinding`."""
@@ -122,6 +151,7 @@ class KeyBindingsRegistry:
             A callable that can be used to unregister the keybinding
         """
         if plat_keybinding := rule._bind_to_current_platform():
+            # list registry
             keybinding = KeyBinding.validate(plat_keybinding)
             if self._filter_keybinding:
                 msg = self._filter_keybinding(keybinding)
@@ -132,12 +162,21 @@ class KeyBindingsRegistry:
                 command_id=id,
                 weight=rule.weight,
                 when=rule.when,
+                source=rule.source,
             )
-            self._keybindings.append(entry)
+
+            # inverse map registry
+            entries = self._keymap[keybinding.to_int()]
+            insort_left(entries, entry)
+
             self.registered.emit()
 
             def _dispose() -> None:
-                self._keybindings.remove(entry)
+                # inverse map registry remove
+                entries.remove(entry)
+                self.unregistered.emit()
+                if len(entries) == 0:
+                    del self._keymap[keybinding.to_int()]
 
             return _dispose
         return None  # pragma: no cover
@@ -145,13 +184,44 @@ class KeyBindingsRegistry:
     def __iter__(self) -> Iterator[_RegisteredKeyBinding]:
         yield from self._keybindings
 
+    def __len__(self) -> int:
+        return sum(len(entries) for entries in self._keymap.values())
+
     def __repr__(self) -> str:
         name = self.__class__.__name__
-        return f"<{name} at {hex(id(self))} ({len(self._keybindings)} bindings)>"
+        return f"<{name} at {hex(id(self))} ({len(self)} bindings)>"
 
-    def get_keybinding(self, key: str) -> _RegisteredKeyBinding | None:
+    def get_keybinding(self, command_id: str) -> _RegisteredKeyBinding | None:
         """Return the first keybinding that matches the given command ID."""
         # TODO: improve me.
-        return next(
-            (entry for entry in self._keybindings if entry.command_id == key), None
-        )
+        matches = (kb for kb in self._keybindings if kb.command_id == command_id)
+        sorted_matches = sorted(matches, key=lambda x: x.source, reverse=True)
+        return next(iter(sorted_matches), None)
+
+    def get_context_prioritized_keybinding(
+        self, key: int, context: Mapping[str, object]
+    ) -> _RegisteredKeyBinding | None:
+        """
+        Return the first keybinding that matches the given key sequence.
+
+        The keybinding should be enabled given the context to be returned.
+
+        Parameters
+        ----------
+        key : int
+            The key sequence that represent the keybinding.
+        context : Mapping[str, object]
+            Variable context to parse the `when` expression to determine if the
+            keybinding is enabled or not.
+
+        Returns
+        -------
+        _RegisteredKeyBinding | None
+            The keybinding found or None otherwise.
+
+        """
+        if key in self._keymap:
+            for entry in reversed(self._keymap[key]):
+                if entry.when is None or entry.when.eval(context):
+                    return entry
+        return None
